@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (c) 2014-2024 OpenRCT2 developers
+ * Copyright (c) 2014-2025 OpenRCT2 developers
  *
  * For a complete list of all authors, please refer to contributors.md
  * Interested in contributing? Visit https://github.com/OpenRCT2/OpenRCT2
@@ -9,25 +9,33 @@
 
 #include "TrackPlaceAction.h"
 
+#include "../Diagnostic.h"
 #include "../GameState.h"
+#include "../core/Money.hpp"
 #include "../core/Numerics.hpp"
 #include "../management/Finance.h"
 #include "../ride/RideData.h"
 #include "../ride/Track.h"
 #include "../ride/TrackData.h"
 #include "../ride/TrackDesign.h"
-#include "../util/Math.hpp"
 #include "../world/ConstructionClearance.h"
+#include "../world/Footpath.h"
 #include "../world/MapAnimation.h"
-#include "../world/Surface.h"
+#include "../world/QuarterTile.h"
+#include "../world/Wall.h"
+#include "../world/tile_element/PathElement.h"
+#include "../world/tile_element/Slope.h"
+#include "../world/tile_element/SurfaceElement.h"
+#include "../world/tile_element/TrackElement.h"
 #include "RideSetSettingAction.h"
 
 using namespace OpenRCT2;
+using namespace OpenRCT2::Numerics;
 using namespace OpenRCT2::TrackMetaData;
 
 TrackPlaceAction::TrackPlaceAction(
-    RideId rideIndex, int32_t trackType, ride_type_t rideType, const CoordsXYZD& origin, int32_t brakeSpeed, int32_t colour,
-    int32_t seatRotation, int32_t liftHillAndAlternativeState, bool fromTrackDesign)
+    RideId rideIndex, OpenRCT2::TrackElemType trackType, ride_type_t rideType, const CoordsXYZD& origin, int32_t brakeSpeed,
+    int32_t colour, int32_t seatRotation, SelectedLiftAndInverted liftHillAndAlternativeState, bool fromTrackDesign)
     : _rideIndex(rideIndex)
     , _trackType(trackType)
     , _rideType(rideType)
@@ -50,7 +58,7 @@ void TrackPlaceAction::AcceptParameters(GameActionParameterVisitor& visitor)
     visitor.Visit("brakeSpeed", _brakeSpeed);
     visitor.Visit("colour", _colour);
     visitor.Visit("seatRotation", _seatRotation);
-    visitor.Visit("trackPlaceFlags", _trackPlaceFlags);
+    visitor.Visit("trackPlaceFlags", _trackPlaceFlags.holder);
     visitor.Visit("isFromTrackDesign", _fromTrackDesign);
 }
 
@@ -64,7 +72,7 @@ void TrackPlaceAction::Serialise(DataSerialiser& stream)
     GameAction::Serialise(stream);
 
     stream << DS_TAG(_rideIndex) << DS_TAG(_trackType) << DS_TAG(_rideType) << DS_TAG(_origin) << DS_TAG(_brakeSpeed)
-           << DS_TAG(_colour) << DS_TAG(_seatRotation) << DS_TAG(_trackPlaceFlags);
+           << DS_TAG(_colour) << DS_TAG(_seatRotation) << DS_TAG(_trackPlaceFlags.holder);
 }
 
 GameActions::Result TrackPlaceAction::Query() const
@@ -91,10 +99,11 @@ GameActions::Result TrackPlaceAction::Query() const
             GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_ERR_VALUE_OUT_OF_RANGE);
     }
 
-    if (_rideType != ride->type && !GetGameState().Cheats.AllowArbitraryRideTypeChanges)
+    auto& gameState = getGameState();
+    if (_rideType != ride->type && !gameState.cheats.allowArbitraryRideTypeChanges)
     {
         return GameActions::Result(
-            GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_NONE);
+            GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, kStringIdNone);
     }
 
     if (_rideType > RIDE_TYPE_COUNT)
@@ -104,6 +113,13 @@ GameActions::Result TrackPlaceAction::Query() const
             GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_ERR_VALUE_OUT_OF_RANGE);
     }
 
+    if (_brakeSpeed > kMaximumTrackSpeed)
+    {
+        LOG_WARNING("Invalid speed for track placement, speed = %d", _brakeSpeed);
+        return GameActions::Result(
+            GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_SPEED_TOO_HIGH);
+    }
+
     auto res = GameActions::Result();
     res.Expenditure = ExpenditureType::RideConstruction;
     res.Position.x = _origin.x + 16;
@@ -112,9 +128,9 @@ GameActions::Result TrackPlaceAction::Query() const
 
     auto resultData = TrackPlaceActionResult{};
 
-    uint32_t rideTypeFlags = ride->GetRideTypeDescriptor().Flags;
+    const auto& rtd = ride->getRideTypeDescriptor();
 
-    if ((ride->lifecycle_flags & RIDE_LIFECYCLE_INDESTRUCTIBLE_TRACK) && _trackType == TrackElemType::EndStation)
+    if ((ride->lifecycleFlags & RIDE_LIFECYCLE_INDESTRUCTIBLE_TRACK) && _trackType == TrackElemType::EndStation)
     {
         return GameActions::Result(
             GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_NOT_ALLOWED_TO_MODIFY_STATION);
@@ -122,7 +138,7 @@ GameActions::Result TrackPlaceAction::Query() const
 
     if (!(GetActionFlags() & GameActions::Flags::AllowWhilePaused))
     {
-        if (GameIsPaused() && !GetGameState().Cheats.BuildInPauseMode)
+        if (GameIsPaused() && !gameState.cheats.buildInPauseMode)
         {
             return GameActions::Result(
                 GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
@@ -130,11 +146,11 @@ GameActions::Result TrackPlaceAction::Query() const
         }
     }
 
-    if (!(rideTypeFlags & RIDE_TYPE_FLAG_FLAT_RIDE))
+    if (!rtd.HasFlag(RtdFlag::isFlatRide))
     {
         if (_trackType == TrackElemType::OnRidePhoto)
         {
-            if (ride->lifecycle_flags & RIDE_LIFECYCLE_ON_RIDE_PHOTO)
+            if (ride->lifecycleFlags & RIDE_LIFECYCLE_ON_RIDE_PHOTO)
             {
                 return GameActions::Result(
                     GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
@@ -143,7 +159,7 @@ GameActions::Result TrackPlaceAction::Query() const
         }
         else if (_trackType == TrackElemType::CableLiftHill)
         {
-            if (ride->lifecycle_flags & RIDE_LIFECYCLE_CABLE_LIFT_HILL_COMPONENT_USED)
+            if (ride->lifecycleFlags & RIDE_LIFECYCLE_CABLE_LIFT_HILL_COMPONENT_USED)
             {
                 return GameActions::Result(
                     GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
@@ -151,12 +167,11 @@ GameActions::Result TrackPlaceAction::Query() const
             }
         }
         // Backwards steep lift hills are allowed, even on roller coasters that do not support forwards steep lift hills.
-        if ((_trackPlaceFlags & CONSTRUCTION_LIFT_HILL_SELECTED)
-            && !ride->GetRideTypeDescriptor().SupportsTrackPiece(TRACK_LIFT_HILL_STEEP)
-            && !GetGameState().Cheats.EnableChainLiftOnAllTrack)
+        if (_trackPlaceFlags.has(LiftHillAndInverted::liftHill) && !rtd.SupportsTrackGroup(TrackGroup::liftHillSteep)
+            && !gameState.cheats.enableChainLiftOnAllTrack)
         {
             const auto& ted = GetTrackElementDescriptor(_trackType);
-            if (ted.Flags & TRACK_ELEM_FLAG_IS_STEEP_UP)
+            if (ted.flags & TRACK_ELEM_FLAG_IS_STEEP_UP)
             {
                 return GameActions::Result(
                     GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
@@ -166,12 +181,12 @@ GameActions::Result TrackPlaceAction::Query() const
     }
 
     const auto& ted = GetTrackElementDescriptor(_trackType);
-    const PreviewTrack* trackBlock = ted.Block;
     uint32_t numElements = 0;
     // First check if any of the track pieces are outside the park
-    for (; trackBlock->index != 0xFF; trackBlock++)
+    for (uint8_t i = 0; i < ted.numSequences; i++)
     {
-        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock->x, trackBlock->y }.Rotate(_origin.direction), 0 };
+        const auto& trackBlock = ted.sequences[i].clearance;
+        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock.x, trackBlock.y }.Rotate(_origin.direction), 0 };
         auto tileCoords = CoordsXYZ{ _origin.x, _origin.y, _origin.z } + rotatedTrack;
 
         if (!LocationValid(tileCoords))
@@ -179,7 +194,7 @@ GameActions::Result TrackPlaceAction::Query() const
             return GameActions::Result(
                 GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_OFF_EDGE_OF_MAP);
         }
-        if (!MapIsLocationOwned(tileCoords) && !GetGameState().Cheats.SandboxMode)
+        if (!MapIsLocationOwned(tileCoords) && !gameState.cheats.sandboxMode)
         {
             return GameActions::Result(
                 GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_LAND_NOT_OWNED_BY_PARK);
@@ -195,9 +210,9 @@ GameActions::Result TrackPlaceAction::Query() const
             STR_TILE_ELEMENT_LIMIT_REACHED);
     }
 
-    if (!GetGameState().Cheats.AllowTrackPlaceInvalidHeights)
+    if (!gameState.cheats.allowTrackPlaceInvalidHeights)
     {
-        if (ted.Flags & TRACK_ELEM_FLAG_STARTS_AT_HALF_HEIGHT)
+        if (ted.flags & TRACK_ELEM_FLAG_STARTS_AT_HALF_HEIGHT)
         {
             if ((_origin.z & 0x0F) != 8)
             {
@@ -216,16 +231,16 @@ GameActions::Result TrackPlaceAction::Query() const
     }
 
     // If that is not the case, then perform the remaining checks
-    trackBlock = ted.Block;
     auto clearanceHeight = rideEntry->Clearance;
 
     money64 costs = 0;
     money64 supportCosts = 0;
-    for (int32_t blockIndex = 0; trackBlock->index != 0xFF; trackBlock++, blockIndex++)
+    for (int32_t blockIndex = 0; blockIndex < ted.numSequences; blockIndex++)
     {
-        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock->x, trackBlock->y }.Rotate(_origin.direction), trackBlock->z };
+        const auto& trackBlock = ted.sequences[blockIndex].clearance;
+        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock.x, trackBlock.y }.Rotate(_origin.direction), trackBlock.z };
         auto mapLoc = CoordsXYZ{ _origin.x, _origin.y, _origin.z } + rotatedTrack;
-        auto quarterTile = trackBlock->var_08.Rotate(_origin.direction);
+        auto quarterTile = trackBlock.quarterTile.Rotate(_origin.direction);
 
         if (mapLoc.z < 16)
         {
@@ -233,10 +248,10 @@ GameActions::Result TrackPlaceAction::Query() const
                 GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_TOO_LOW);
         }
 
-        int32_t baseZ = Floor2(mapLoc.z, COORDS_Z_STEP);
+        int32_t baseZ = floor2(mapLoc.z, kCoordsZStep);
 
-        int32_t clearanceZ = trackBlock->ClearanceZ;
-        if (trackBlock->flags & RCT_PREVIEW_TRACK_FLAG_IS_VERTICAL && clearanceHeight > 24)
+        int32_t clearanceZ = trackBlock.clearanceZ;
+        if (trackBlock.flags & RCT_PREVIEW_TRACK_FLAG_IS_VERTICAL && clearanceHeight > 24)
         {
             clearanceZ += 24;
         }
@@ -245,7 +260,7 @@ GameActions::Result TrackPlaceAction::Query() const
             clearanceZ += clearanceHeight;
         }
 
-        clearanceZ = Floor2(clearanceZ, COORDS_Z_STEP) + baseZ;
+        clearanceZ = floor2(clearanceZ, kCoordsZStep) + baseZ;
 
         if (clearanceZ > MAX_TRACK_HEIGHT)
         {
@@ -253,10 +268,9 @@ GameActions::Result TrackPlaceAction::Query() const
                 GameActions::Status::InvalidParameters, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE, STR_TOO_HIGH);
         }
 
-        uint8_t crossingMode = (ride->GetRideTypeDescriptor().HasFlag(RIDE_TYPE_FLAG_SUPPORTS_LEVEL_CROSSINGS)
-                                && _trackType == TrackElemType::Flat)
-            ? CREATE_CROSSING_MODE_TRACK_OVER_PATH
-            : CREATE_CROSSING_MODE_NONE;
+        auto crossingMode = (rtd.HasFlag(RtdFlag::supportsLevelCrossings) && _trackType == TrackElemType::Flat)
+            ? CreateCrossingMode::trackOverPath
+            : CreateCrossingMode::none;
         auto canBuild = MapCanConstructWithClearAt(
             { mapLoc, baseZ, clearanceZ }, &MapPlaceNonSceneryClearFunc, quarterTile, GetFlags(), crossingMode);
         if (canBuild.Error != GameActions::Status::Ok)
@@ -268,7 +282,7 @@ GameActions::Result TrackPlaceAction::Query() const
 
         const auto clearanceData = canBuild.GetData<ConstructClearResult>();
         uint8_t mapGroundFlags = clearanceData.GroundFlags & (ELEMENT_IS_ABOVE_GROUND | ELEMENT_IS_UNDERGROUND);
-        if (!(ted.Flags & TRACK_ELEM_FLAG_CAN_BE_PARTLY_UNDERGROUND))
+        if (!(ted.flags & TRACK_ELEM_FLAG_CAN_BE_PARTLY_UNDERGROUND))
         {
             if (resultData.GroundFlags != 0 && (resultData.GroundFlags & mapGroundFlags) == 0)
             {
@@ -279,7 +293,7 @@ GameActions::Result TrackPlaceAction::Query() const
         }
 
         resultData.GroundFlags = mapGroundFlags;
-        if (ted.Flags & TRACK_ELEM_FLAG_ONLY_ABOVE_GROUND)
+        if (ted.flags & TRACK_ELEM_FLAG_ONLY_ABOVE_GROUND)
         {
             if (resultData.GroundFlags & ELEMENT_IS_UNDERGROUND)
             {
@@ -289,7 +303,7 @@ GameActions::Result TrackPlaceAction::Query() const
             }
         }
 
-        if (ted.Flags & TRACK_ELEM_FLAG_ONLY_UNDERWATER)
+        if (ted.flags & TRACK_ELEM_FLAG_ONLY_UNDERWATER)
         { // No element has this flag
             if (clearanceData.GroundFlags & ELEMENT_IS_UNDERWATER)
             {
@@ -299,14 +313,14 @@ GameActions::Result TrackPlaceAction::Query() const
             }
         }
 
-        if (clearanceData.GroundFlags & ELEMENT_IS_UNDERWATER && !GetGameState().Cheats.DisableClearanceChecks)
+        if (clearanceData.GroundFlags & ELEMENT_IS_UNDERWATER && !gameState.cheats.disableClearanceChecks)
         {
             return GameActions::Result(
                 GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
                 STR_RIDE_CANT_BUILD_THIS_UNDERWATER);
         }
 
-        if ((rideTypeFlags & RIDE_TYPE_FLAG_TRACK_MUST_BE_ON_WATER) && !_trackDesignDrawingPreview)
+        if (rtd.HasFlag(RtdFlag::trackMustBeOnWater) && !_trackDesignDrawingPreview)
         {
             auto surfaceElement = MapGetSurfaceElementAt(mapLoc);
             if (surfaceElement == nullptr)
@@ -330,12 +344,12 @@ GameActions::Result TrackPlaceAction::Query() const
                     GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
                     STR_CAN_ONLY_BUILD_THIS_ON_WATER);
             }
-            waterHeight -= LAND_HEIGHT_STEP;
+            waterHeight -= kLandHeightStep;
             if (waterHeight == surfaceElement->GetBaseZ())
             {
-                uint8_t slope = surfaceElement->GetSlope() & TILE_ELEMENT_SLOPE_ALL_CORNERS_UP;
-                if (slope == TILE_ELEMENT_SLOPE_W_CORNER_DN || slope == TILE_ELEMENT_SLOPE_S_CORNER_DN
-                    || slope == TILE_ELEMENT_SLOPE_E_CORNER_DN || slope == TILE_ELEMENT_SLOPE_N_CORNER_DN)
+                uint8_t slope = surfaceElement->GetSlope() & kTileSlopeRaisedCornersMask;
+                if (slope == kTileSlopeWCornerDown || slope == kTileSlopeSCornerDown || slope == kTileSlopeECornerDown
+                    || slope == kTileSlopeNCornerDown)
                 {
                     return GameActions::Result(
                         GameActions::Status::Disallowed, STR_RIDE_CONSTRUCTION_CANT_CONSTRUCT_THIS_HERE,
@@ -344,8 +358,8 @@ GameActions::Result TrackPlaceAction::Query() const
             }
         }
 
-        int32_t entranceDirections = std::get<0>(ted.SequenceProperties);
-        if ((entranceDirections & TRACK_SEQUENCE_FLAG_ORIGIN) && trackBlock->index == 0)
+        int32_t entranceDirections = ted.sequences[0].flags;
+        if ((entranceDirections & TRACK_SEQUENCE_FLAG_ORIGIN) && blockIndex == 0)
         {
             const auto addElementResult = TrackAddStationElement(
                 { mapLoc, baseZ, _origin.direction }, _rideIndex, 0, _fromTrackDesign);
@@ -365,24 +379,23 @@ GameActions::Result TrackPlaceAction::Query() const
                 STR_ERR_SURFACE_ELEMENT_NOT_FOUND);
         }
 
-        if (!GetGameState().Cheats.DisableSupportLimits)
+        if (!gameState.cheats.disableSupportLimits)
         {
             int32_t ride_height = clearanceZ - surfaceElement->GetBaseZ();
             if (ride_height >= 0)
             {
                 uint16_t maxHeight;
 
-                if (ride->GetRideTypeDescriptor().HasFlag(RIDE_TYPE_FLAG_LIST_VEHICLES_SEPARATELY)
-                    && rideEntry->max_height != 0)
+                if (rtd.HasFlag(RtdFlag::listVehiclesSeparately) && rideEntry->max_height != 0)
                 {
                     maxHeight = rideEntry->max_height;
                 }
                 else
                 {
-                    maxHeight = ride->GetRideTypeDescriptor().Heights.MaxHeight;
+                    maxHeight = rtd.Heights.MaxHeight;
                 }
 
-                ride_height /= COORDS_Z_PER_TINY_Z;
+                ride_height /= kCoordsZPerTinyZ;
                 if (ride_height > maxHeight && !_trackDesignDrawingPreview)
                 {
                     return GameActions::Result(
@@ -395,14 +408,14 @@ GameActions::Result TrackPlaceAction::Query() const
         int32_t supportHeight = baseZ - surfaceElement->GetBaseZ();
         if (supportHeight < 0)
         {
-            supportHeight = (10 * COORDS_Z_STEP);
+            supportHeight = (10 * kCoordsZStep);
         }
 
-        supportCosts += ((supportHeight / (2 * COORDS_Z_STEP)) * ride->GetRideTypeDescriptor().BuildCosts.SupportPrice);
+        supportCosts += ((supportHeight / (2 * kCoordsZStep)) * rtd.BuildCosts.SupportPrice);
     }
 
-    money64 price = ride->GetRideTypeDescriptor().BuildCosts.TrackPrice;
-    price *= ted.PriceModifier;
+    money64 price = rtd.BuildCosts.TrackPrice;
+    price *= ted.priceModifier;
 
     price >>= 16;
     res.Cost = costs + supportCosts + price;
@@ -437,27 +450,27 @@ GameActions::Result TrackPlaceAction::Execute() const
 
     auto resultData = TrackPlaceActionResult{};
 
-    uint32_t rideTypeFlags = ride->GetRideTypeDescriptor().Flags;
+    const auto& rtd = ride->getRideTypeDescriptor();
 
     const auto& ted = GetTrackElementDescriptor(_trackType);
-    const auto& wallEdges = ted.SequenceElementAllowedWallEdges;
 
     money64 costs = 0;
     money64 supportCosts = 0;
-    const PreviewTrack* trackBlock = ted.Block;
+    const auto& block0 = ted.sequences[0].clearance;
     auto clearanceHeight = rideEntry->Clearance;
     CoordsXYZ originLocation = CoordsXYZ{ _origin.x, _origin.y, _origin.z }
-        + CoordsXYZ{ CoordsXY{ trackBlock->x, trackBlock->y }.Rotate(_origin.direction), trackBlock->z };
-    for (int32_t blockIndex = 0; trackBlock->index != 0xFF; trackBlock++, blockIndex++)
+        + CoordsXYZ{ CoordsXY{ block0.x, block0.y }.Rotate(_origin.direction), block0.z };
+    for (int32_t blockIndex = 0; blockIndex < ted.numSequences; blockIndex++)
     {
-        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock->x, trackBlock->y }.Rotate(_origin.direction), trackBlock->z };
+        const auto& trackBlock = ted.sequences[blockIndex].clearance;
+        auto rotatedTrack = CoordsXYZ{ CoordsXY{ trackBlock.x, trackBlock.y }.Rotate(_origin.direction), trackBlock.z };
         auto mapLoc = CoordsXYZ{ _origin.x, _origin.y, _origin.z } + rotatedTrack;
 
-        auto quarterTile = trackBlock->var_08.Rotate(_origin.direction);
+        auto quarterTile = trackBlock.quarterTile.Rotate(_origin.direction);
 
-        int32_t baseZ = Floor2(mapLoc.z, COORDS_Z_STEP);
-        int32_t clearanceZ = trackBlock->ClearanceZ;
-        if (trackBlock->flags & RCT_PREVIEW_TRACK_FLAG_IS_VERTICAL && clearanceHeight > 24)
+        int32_t baseZ = floor2(mapLoc.z, kCoordsZStep);
+        int32_t clearanceZ = trackBlock.clearanceZ;
+        if (trackBlock.flags & RCT_PREVIEW_TRACK_FLAG_IS_VERTICAL && clearanceHeight > 24)
         {
             clearanceZ += 24;
         }
@@ -466,13 +479,12 @@ GameActions::Result TrackPlaceAction::Execute() const
             clearanceZ += clearanceHeight;
         }
 
-        clearanceZ = Floor2(clearanceZ, COORDS_Z_STEP) + baseZ;
+        clearanceZ = floor2(clearanceZ, kCoordsZStep) + baseZ;
         const auto mapLocWithClearance = CoordsXYRangedZ(mapLoc, baseZ, clearanceZ);
 
-        uint8_t crossingMode = (ride->GetRideTypeDescriptor().HasFlag(RIDE_TYPE_FLAG_SUPPORTS_LEVEL_CROSSINGS)
-                                && _trackType == TrackElemType::Flat)
-            ? CREATE_CROSSING_MODE_TRACK_OVER_PATH
-            : CREATE_CROSSING_MODE_NONE;
+        auto crossingMode = (rtd.HasFlag(RtdFlag::supportsLevelCrossings) && _trackType == TrackElemType::Flat)
+            ? CreateCrossingMode::trackOverPath
+            : CreateCrossingMode::none;
         auto canBuild = MapCanConstructWithClearAt(
             mapLocWithClearance, &MapPlaceNonSceneryClearFunc, quarterTile, GetFlags() | GAME_COMMAND_FLAG_APPLY, crossingMode);
         if (canBuild.Error != GameActions::Status::Ok)
@@ -483,7 +495,7 @@ GameActions::Result TrackPlaceAction::Execute() const
         costs += canBuild.Cost;
 
         // When building a level crossing, remove any pre-existing path furniture.
-        if (crossingMode == CREATE_CROSSING_MODE_TRACK_OVER_PATH && !(GetFlags() & GAME_COMMAND_FLAG_GHOST))
+        if (crossingMode == CreateCrossingMode::trackOverPath && !(GetFlags() & GAME_COMMAND_FLAG_GHOST))
         {
             auto footpathElement = MapGetFootpathElement(mapLoc);
             if (footpathElement != nullptr && footpathElement->HasAddition())
@@ -492,20 +504,21 @@ GameActions::Result TrackPlaceAction::Execute() const
             }
         }
 
-        if (!(GetFlags() & GAME_COMMAND_FLAG_GHOST) && !GetGameState().Cheats.DisableClearanceChecks)
+        auto& gameState = getGameState();
+        if (!(GetFlags() & GAME_COMMAND_FLAG_GHOST) && !gameState.cheats.disableClearanceChecks)
         {
             FootpathRemoveLitter(mapLoc);
-            if (rideTypeFlags & RIDE_TYPE_FLAG_TRACK_NO_WALLS)
+            if (rtd.HasFlag(RtdFlag::noWallsAroundTrack))
             {
                 WallRemoveAt(mapLocWithClearance);
             }
             else
             {
                 // Remove walls in the directions this track intersects
-                uint8_t intersectingDirections = wallEdges[blockIndex];
+                uint8_t intersectingDirections = ted.sequences[blockIndex].allowedWallEdges;
                 intersectingDirections ^= 0x0F;
                 intersectingDirections = Numerics::rol4(intersectingDirections, _origin.direction);
-                for (int32_t i = 0; i < NumOrthogonalDirections; i++)
+                for (int32_t i = 0; i < kNumOrthogonalDirections; i++)
                 {
                     if (intersectingDirections & (1 << i))
                     {
@@ -517,7 +530,7 @@ GameActions::Result TrackPlaceAction::Execute() const
 
         const auto clearanceData = canBuild.GetData<ConstructClearResult>();
         uint8_t mapGroundFlags = clearanceData.GroundFlags & (ELEMENT_IS_ABOVE_GROUND | ELEMENT_IS_UNDERGROUND);
-        if (!(ted.Flags & TRACK_ELEM_FLAG_CAN_BE_PARTLY_UNDERGROUND))
+        if (!(ted.flags & TRACK_ELEM_FLAG_CAN_BE_PARTLY_UNDERGROUND))
         {
             if (resultData.GroundFlags != 0 && (resultData.GroundFlags & mapGroundFlags) == 0)
             {
@@ -541,23 +554,23 @@ GameActions::Result TrackPlaceAction::Execute() const
         int32_t supportHeight = baseZ - surfaceElement->GetBaseZ();
         if (supportHeight < 0)
         {
-            supportHeight = (10 * COORDS_Z_STEP);
+            supportHeight = (10 * kCoordsZStep);
         }
 
-        supportCosts += (supportHeight / (2 * COORDS_Z_STEP)) * ride->GetRideTypeDescriptor().BuildCosts.SupportPrice;
+        supportCosts += (supportHeight / (2 * kCoordsZStep)) * rtd.BuildCosts.SupportPrice;
 
         int32_t entranceDirections = 0;
-        if (!ride->overall_view.IsNull())
+        if (!ride->overallView.IsNull())
         {
             if (!(GetFlags() & GAME_COMMAND_FLAG_NO_SPEND))
             {
-                entranceDirections = std::get<0>(ted.SequenceProperties);
+                entranceDirections = ted.sequences[0].flags;
             }
         }
 
-        if (entranceDirections & TRACK_SEQUENCE_FLAG_ORIGIN || ride->overall_view.IsNull())
+        if (entranceDirections & TRACK_SEQUENCE_FLAG_ORIGIN || ride->overallView.IsNull())
         {
-            ride->overall_view = mapLoc;
+            ride->overallView = mapLoc;
         }
 
         auto* trackElement = TileElementInsert<TrackElement>(mapLoc, quarterTile.GetBaseQuarterOccupied());
@@ -571,8 +584,8 @@ GameActions::Result TrackPlaceAction::Execute() const
 
         trackElement->SetClearanceZ(clearanceZ);
         trackElement->SetDirection(_origin.direction);
-        trackElement->SetHasChain(_trackPlaceFlags & CONSTRUCTION_LIFT_HILL_SELECTED);
-        trackElement->SetSequenceIndex(trackBlock->index);
+        trackElement->SetHasChain(_trackPlaceFlags.has(LiftHillAndInverted::liftHill));
+        trackElement->SetSequenceIndex(blockIndex);
         trackElement->SetRideIndex(_rideIndex);
         trackElement->SetTrackType(_trackType);
         trackElement->SetRideType(_rideType);
@@ -581,53 +594,49 @@ GameActions::Result TrackPlaceAction::Execute() const
         switch (_trackType)
         {
             case TrackElemType::Waterfall:
-                MapAnimationCreate(MAP_ANIMATION_TYPE_TRACK_WATERFALL, CoordsXYZ{ mapLoc, trackElement->GetBaseZ() });
-                break;
             case TrackElemType::Rapids:
-                MapAnimationCreate(MAP_ANIMATION_TYPE_TRACK_RAPIDS, CoordsXYZ{ mapLoc, trackElement->GetBaseZ() });
-                break;
             case TrackElemType::Whirlpool:
-                MapAnimationCreate(MAP_ANIMATION_TYPE_TRACK_WHIRLPOOL, CoordsXYZ{ mapLoc, trackElement->GetBaseZ() });
-                break;
             case TrackElemType::SpinningTunnel:
-                MapAnimationCreate(MAP_ANIMATION_TYPE_TRACK_SPINNINGTUNNEL, CoordsXYZ{ mapLoc, trackElement->GetBaseZ() });
+                MapAnimations::MarkTileForInvalidation(TileCoordsXY(mapLoc));
                 break;
             case TrackElemType::Brakes:
             case TrackElemType::DiagBrakes:
                 trackElement->SetBrakeClosed(true);
                 break;
+            default:
+                break;
         }
         if (TrackTypeHasSpeedSetting(_trackType))
         {
             trackElement->SetBrakeBoosterSpeed(_brakeSpeed);
         }
 
-        if (ride->GetRideTypeDescriptor().HasFlag(RIDE_TYPE_FLAG_HAS_LANDSCAPE_DOORS))
+        if (rtd.HasFlag(RtdFlag::hasLandscapeDoors))
         {
-            trackElement->SetDoorAState(LANDSCAPE_DOOR_CLOSED);
-            trackElement->SetDoorBState(LANDSCAPE_DOOR_CLOSED);
+            trackElement->SetDoorAState(kLandEdgeDoorFrameClosed);
+            trackElement->SetDoorBState(kLandEdgeDoorFrameClosed);
         }
         else
         {
             trackElement->SetSeatRotation(_seatRotation);
         }
 
-        if (_trackPlaceFlags & RIDE_TYPE_ALTERNATIVE_TRACK_TYPE)
+        if (_trackPlaceFlags.has(LiftHillAndInverted::inverted))
         {
             trackElement->SetInverted(true);
         }
-        trackElement->SetColourScheme(_colour);
+        trackElement->SetColourScheme(static_cast<RideColourScheme>(_colour));
 
-        entranceDirections = std::get<0>(ted.SequenceProperties);
+        entranceDirections = ted.sequences[0].flags;
         if (entranceDirections & TRACK_SEQUENCE_FLAG_CONNECTS_TO_PATH)
         {
-            uint8_t availableDirections = entranceDirections & 0x0F;
+            uint32_t availableDirections = entranceDirections & 0x0F;
             if (availableDirections != 0)
             {
-                if (!(GetFlags() & GAME_COMMAND_FLAG_GHOST) && !GetGameState().Cheats.DisableClearanceChecks)
+                if (!(GetFlags() & GAME_COMMAND_FLAG_GHOST) && !gameState.cheats.disableClearanceChecks)
                 {
-                    for (int32_t chosenDirection = UtilBitScanForward(availableDirections); chosenDirection != -1;
-                         chosenDirection = UtilBitScanForward(availableDirections))
+                    for (int32_t chosenDirection = Numerics::bitScanForward(availableDirections); chosenDirection != -1;
+                         chosenDirection = Numerics::bitScanForward(availableDirections))
                     {
                         availableDirections &= ~(1 << chosenDirection);
                         CoordsXY tempLoc{ mapLoc.x, mapLoc.y };
@@ -646,17 +655,17 @@ GameActions::Result TrackPlaceAction::Execute() const
         // However, ghost tiles from track designs need to modify station data to display properly
         if (entranceDirections & TRACK_SEQUENCE_FLAG_ORIGIN && (!(GetFlags() & GAME_COMMAND_FLAG_GHOST) || _fromTrackDesign))
         {
-            if (trackBlock->index == 0)
+            if (blockIndex == 0)
             {
                 TrackAddStationElement({ mapLoc, _origin.direction }, _rideIndex, GAME_COMMAND_FLAG_APPLY, _fromTrackDesign);
             }
-            ride->ValidateStations();
-            ride->UpdateMaxVehicles();
+            ride->validateStations();
+            ride->updateMaxVehicles();
         }
 
         auto* tileElement = trackElement->as<TileElement>();
 
-        if (rideTypeFlags & RIDE_TYPE_FLAG_TRACK_MUST_BE_ON_WATER)
+        if (rtd.HasFlag(RtdFlag::trackMustBeOnWater))
         {
             auto* waterSurfaceElement = MapGetSurfaceElementAt(mapLoc);
             if (waterSurfaceElement != nullptr)
@@ -666,7 +675,7 @@ GameActions::Result TrackPlaceAction::Execute() const
             }
         }
 
-        if (!GetGameState().Cheats.DisableClearanceChecks || !(GetFlags() & GAME_COMMAND_FLAG_GHOST))
+        if (!gameState.cheats.disableClearanceChecks || !(GetFlags() & GAME_COMMAND_FLAG_GHOST))
         {
             FootpathConnectEdges(mapLoc, tileElement, GetFlags());
         }
@@ -680,33 +689,35 @@ GameActions::Result TrackPlaceAction::Execute() const
         switch (_trackType)
         {
             case TrackElemType::OnRidePhoto:
-                ride->lifecycle_flags |= RIDE_LIFECYCLE_ON_RIDE_PHOTO;
+                ride->lifecycleFlags |= RIDE_LIFECYCLE_ON_RIDE_PHOTO;
                 break;
             case TrackElemType::CableLiftHill:
-                ride->lifecycle_flags |= RIDE_LIFECYCLE_CABLE_LIFT_HILL_COMPONENT_USED;
-                ride->CableLiftLoc = originLocation;
+                ride->lifecycleFlags |= RIDE_LIFECYCLE_CABLE_LIFT_HILL_COMPONENT_USED;
+                ride->cableLiftLoc = originLocation;
                 break;
             case TrackElemType::DiagBlockBrakes:
             case TrackElemType::BlockBrakes:
             {
-                ride->num_block_brakes++;
-                ride->window_invalidate_flags |= RIDE_INVALIDATE_RIDE_OPERATING;
+                ride->numBlockBrakes++;
+                ride->windowInvalidateFlags |= RIDE_INVALIDATE_RIDE_OPERATING;
 
                 // change the current mode to its circuit blocked equivalent
-                RideMode newMode = RideMode::ContinuousCircuitBlockSectioned;
-                if (ride->mode == RideMode::PoweredLaunch)
+                RideMode newMode = RideMode::continuousCircuitBlockSectioned;
+                if (ride->mode == RideMode::poweredLaunch)
                 {
-                    if (ride->GetRideTypeDescriptor().SupportsRideMode(RideMode::PoweredLaunchBlockSectioned)
-                        || GetGameState().Cheats.ShowAllOperatingModes)
-                        newMode = RideMode::PoweredLaunchBlockSectioned;
+                    if (rtd.SupportsRideMode(RideMode::poweredLaunchBlockSectioned)
+                        || getGameState().cheats.showAllOperatingModes)
+                        newMode = RideMode::poweredLaunchBlockSectioned;
                     else
-                        newMode = RideMode::PoweredLaunch;
+                        newMode = RideMode::poweredLaunch;
                 }
 
                 auto rideSetSetting = RideSetSettingAction(ride->id, RideSetSetting::Mode, static_cast<uint8_t>(newMode));
                 GameActions::ExecuteNested(&rideSetSetting);
                 break;
             }
+            default:
+                break;
         }
 
         switch (_trackType)
@@ -715,17 +726,19 @@ GameActions::Result TrackPlaceAction::Execute() const
             case TrackElemType::Up60ToFlat:
             case TrackElemType::DiagUp25ToFlat:
             case TrackElemType::DiagUp60ToFlat:
-                if (!(_trackPlaceFlags & CONSTRUCTION_LIFT_HILL_SELECTED))
+                if (!_trackPlaceFlags.has(LiftHillAndInverted::liftHill))
                     break;
                 [[fallthrough]];
             case TrackElemType::CableLiftHill:
-                ride->num_block_brakes++;
+                ride->numBlockBrakes++;
+                break;
+            default:
                 break;
         }
     }
 
-    money64 price = ride->GetRideTypeDescriptor().BuildCosts.TrackPrice;
-    price *= ted.PriceModifier;
+    money64 price = rtd.BuildCosts.TrackPrice;
+    price *= ted.priceModifier;
 
     price >>= 16;
     res.Cost = costs + supportCosts + price;
@@ -737,9 +750,10 @@ GameActions::Result TrackPlaceAction::Execute() const
 bool TrackPlaceAction::CheckMapCapacity(int16_t numTiles) const
 {
     const auto& ted = GetTrackElementDescriptor(_trackType);
-    for (const PreviewTrack* trackBlock = ted.Block; trackBlock->index != 0xFF; trackBlock++)
+    for (uint8_t i = 0; i < ted.numSequences; i++)
     {
-        auto rotatedTrack = CoordsXY{ trackBlock->x, trackBlock->y }.Rotate(_origin.direction);
+        const auto& trackBlock = ted.sequences[i].clearance;
+        auto rotatedTrack = CoordsXY{ trackBlock.x, trackBlock.y }.Rotate(_origin.direction);
 
         auto tileCoords = CoordsXY{ _origin.x, _origin.y } + rotatedTrack;
         if (!MapCheckCapacityAndReorganise(tileCoords, numTiles))
